@@ -14,6 +14,7 @@ use std::sync::Arc;
 use futures_util::{self, future};
 use proto::error::ProtoResult;
 use proto::op::Query;
+use proto::rr::domain::usage::ONION;
 use proto::rr::domain::TryParseIp;
 use proto::rr::{IntoName, Name, Record, RecordType};
 use proto::xfer::{DnsRequestOptions, RetryDnsHandle};
@@ -70,6 +71,7 @@ pub struct AsyncResolver<C: DnsHandle<Error = ResolveError>, P: ConnectionProvid
 
 /// An AsyncResolver used with Tokio
 #[cfg(feature = "tokio-runtime")]
+#[cfg_attr(docsrs, doc(cfg(feature = "tokio-runtime")))]
 pub type TokioAsyncResolver = AsyncResolver<TokioConnection, TokioConnectionProvider>;
 
 macro_rules! lookup_fn {
@@ -89,15 +91,10 @@ macro_rules! lookup_fn {
                 }
             };
 
-            self.inner_lookup(
-                name,
-                $r,
-                DnsRequestOptions {
-                    use_edns: self.options.edns0,
-                    ..Default::default()
-                },
-            )
-            .await
+            let mut request_opts = DnsRequestOptions::default();
+            request_opts.use_edns = self.options.edns0;
+
+            self.inner_lookup(name, $r, request_opts).await
         }
     };
     ($p:ident, $l:ty, $r:path, $t:ty) => {
@@ -115,6 +112,7 @@ macro_rules! lookup_fn {
 }
 
 #[cfg(feature = "tokio-runtime")]
+#[cfg_attr(docsrs, doc(cfg(feature = "tokio-runtime")))]
 impl TokioAsyncResolver {
     /// Construct a new Tokio based `AsyncResolver` with the provided configuration.
     ///
@@ -138,6 +136,10 @@ impl TokioAsyncResolver {
     /// This will use `/etc/resolv.conf` on Unix OSes and the registry on Windows.
     #[cfg(any(unix, target_os = "windows"))]
     #[cfg(feature = "system-config")]
+    #[cfg_attr(
+        docsrs,
+        doc(cfg(all(feature = "system-config", any(unix, target_os = "windows"))))
+    )]
     pub fn tokio_from_system_conf() -> Result<Self, ResolveError> {
         Self::from_system_conf(TokioHandle)
     }
@@ -146,7 +148,7 @@ impl TokioAsyncResolver {
 impl<R: RuntimeProvider> AsyncResolver<GenericConnection, GenericConnectionProvider<R>> {
     /// Construct a new generic `AsyncResolver` with the provided configuration.
     ///
-    /// see [crate::trust_dns_resolver::TokioAsyncResolver::tokio] instead.
+    /// see [TokioAsyncResolver::tokio(..)] instead.
     ///
     /// # Arguments
     ///
@@ -164,7 +166,7 @@ impl<R: RuntimeProvider> AsyncResolver<GenericConnection, GenericConnectionProvi
         options: ResolverOpts,
         runtime: R::Handle,
     ) -> Result<Self, ResolveError> {
-        AsyncResolver::<GenericConnection, GenericConnectionProvider<R>>::new_with_conn(
+        Self::new_with_conn(
             config,
             options,
             GenericConnectionProvider::<R>::new(runtime),
@@ -173,14 +175,23 @@ impl<R: RuntimeProvider> AsyncResolver<GenericConnection, GenericConnectionProvi
 
     /// Constructs a new Resolver with the system configuration.
     ///
-    /// see [crate::trust_dns_resolver::TokioAsyncResolver::tokio_from_system_conf] instead.
+    /// see [TokioAsyncResolver::tokio_from_system_conf(..)] instead.
     ///
     /// This will use `/etc/resolv.conf` on Unix OSes and the registry on Windows.
     #[cfg(any(unix, target_os = "windows"))]
     #[cfg(feature = "system-config")]
+    #[cfg_attr(
+        docsrs,
+        doc(cfg(all(feature = "system-config", any(unix, target_os = "windows"))))
+    )]
     pub fn from_system_conf(runtime: R::Handle) -> Result<Self, ResolveError> {
         Self::from_system_conf_with_provider(GenericConnectionProvider::<R>::new(runtime))
     }
+
+    /// Flushes/Removes all entries from the cache
+    pub async fn clear_cache(&mut self) {
+        self.client_cache.clear_cache();
+    }
 }
 
 impl<C: DnsHandle<Error = ResolveError>, P: ConnectionProvider<Conn = C>> AsyncResolver<C, P> {
@@ -231,7 +242,7 @@ impl<C: DnsHandle<Error = ResolveError>, P: ConnectionProvider<Conn = C>> AsyncR
 
         trace!("handle passed back");
         let lru = DnsLru::new(options.cache_size, dns_lru::TtlConfig::from_opts(&options));
-        Ok(AsyncResolver {
+        Ok(Self {
             config,
             options,
             client_cache: CachingClient::with_cache(lru, either, options.preserve_intermediates),
@@ -244,6 +255,10 @@ impl<C: DnsHandle<Error = ResolveError>, P: ConnectionProvider<Conn = C>> AsyncR
     /// This will use `/etc/resolv.conf` on Unix OSes and the registry on Windows.
     #[cfg(any(unix, target_os = "windows"))]
     #[cfg(feature = "system-config")]
+    #[cfg_attr(
+        docsrs,
+        doc(cfg(all(feature = "system-config", any(unix, target_os = "windows"))))
+    )]
     pub(crate) fn from_system_conf_with_provider(conn_provider: P) -> Result<Self, ResolveError> {
         let (config, options) = super::system_conf::read_system_conf()?;
         Self::new_with_conn(config, options, conn_provider)
@@ -289,7 +304,17 @@ impl<C: DnsHandle<Error = ResolveError>, P: ConnectionProvider<Conn = C>> AsyncR
 
     fn build_names(&self, name: Name) -> Vec<Name> {
         // if it's fully qualified, we can short circuit the lookup logic
-        if name.is_fqdn() {
+        if name.is_fqdn()
+            || ONION.zone_of(&name)
+                && name
+                    .trim_to(2)
+                    .iter()
+                    .next()
+                    .map(|name| name.len() == 56) // size of onion v3 address
+                    .unwrap_or(false)
+        {
+            // if already fully qualified, or if onion address, don't assume it might be a
+            // sub-domain
             vec![name]
         } else {
             // Otherwise we have to build the search list
@@ -308,12 +333,26 @@ impl<C: DnsHandle<Error = ResolveError>, P: ConnectionProvider<Conn = C>> AsyncR
 
             for search in self.config.search().iter().rev() {
                 let name_search = name.clone().append_domain(search);
-                Self::push_name(name_search, &mut names);
+
+                match name_search {
+                    Ok(name_search) => Self::push_name(name_search, &mut names),
+                    Err(e) => debug!(
+                        "Not adding {} to {} for search due to error: {}",
+                        search, name, e
+                    ),
+                }
             }
 
             if let Some(domain) = self.config.domain() {
                 let name_search = name.clone().append_domain(domain);
-                Self::push_name(name_search, &mut names);
+
+                match name_search {
+                    Ok(name_search) => Self::push_name(name_search, &mut names),
+                    Err(e) => debug!(
+                        "Not adding {} to {} for search due to error: {}",
+                        domain, name, e
+                    ),
+                }
             }
 
             // this is the direct name lookup
@@ -392,7 +431,7 @@ impl<C: DnsHandle<Error = ResolveError>, P: ConnectionProvider<Conn = C>> AsyncR
             self.client_cache.clone(),
             DnsRequestOptions::default(),
             hosts,
-            finally_ip_addr.map(Record::into_data),
+            finally_ip_addr.and_then(Record::into_data),
         )
         .await
     }
@@ -425,7 +464,8 @@ impl<C: DnsHandle<Error = ResolveError>, P: ConnectionProvider<Conn = C>> fmt::D
 
 /// Unit tests compatible with different runtime.
 #[cfg(any(test, feature = "testing"))]
-#[allow(dead_code)]
+#[cfg_attr(docsrs, doc(cfg(feature = "testing")))]
+#[allow(dead_code, unreachable_pub)]
 pub mod testing {
     use std::{net::*, str::FromStr};
 
@@ -552,6 +592,7 @@ pub mod testing {
 
     /// Test IP lookup from URLs with DNSSec validation.
     #[cfg(feature = "dnssec")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "dnssec")))]
     pub fn sec_lookup_test<E: Executor + Send + 'static, R: RuntimeProvider>(
         mut exec: E,
         handle: R::Handle,
@@ -562,6 +603,7 @@ pub mod testing {
             ResolverConfig::default(),
             ResolverOpts {
                 validate: true,
+                try_tcp_on_error: true,
                 ..ResolverOpts::default()
             },
             handle,
@@ -591,6 +633,7 @@ pub mod testing {
     /// Test IP lookup from domains that exist but unsigned with DNSSec validation.
     #[allow(deprecated)]
     #[cfg(feature = "dnssec")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "dnssec")))]
     pub fn sec_lookup_fails_test<E: Executor + Send + 'static, R: RuntimeProvider>(
         mut exec: E,
         handle: R::Handle,
@@ -609,8 +652,7 @@ pub mod testing {
         .expect("failed to create resolver");
 
         // needs to be a domain that exists, but is not signed (eventually this will be)
-        let name = Name::from_str("www.trust-dns.org.").unwrap();
-        let response = exec.block_on(resolver.lookup_ip("www.trust-dns.org."));
+        let response = exec.block_on(resolver.lookup_ip("trust-dns.org."));
 
         assert!(response.is_err());
         let error = response.unwrap_err();
@@ -618,6 +660,7 @@ pub mod testing {
         use proto::error::{ProtoError, ProtoErrorKind};
 
         let error_str = format!("{}", error);
+        let name = Name::from_str("trust-dns.org.").unwrap();
         let expected_str = format!(
             "{}",
             ResolveError::from(ProtoError::from(ProtoErrorKind::RrsigsNotPresent {
@@ -632,8 +675,9 @@ pub mod testing {
         }
     }
 
-    #[cfg(feature = "system-config")]
     /// Test AsyncResolver created from system configuration with IP lookup.
+    #[cfg(feature = "system-config")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "system-config")))]
     pub fn system_lookup_test<E: Executor + Send + 'static, R: RuntimeProvider>(
         mut exec: E,
         handle: R::Handle,
@@ -663,8 +707,9 @@ pub mod testing {
         }
     }
 
-    #[cfg(feature = "system-config")]
     /// Test AsyncResolver created from system configuration with host lookups.
+    #[cfg(feature = "system-config")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "system-config")))]
     pub fn hosts_lookup_test<E: Executor + Send + 'static, R: RuntimeProvider>(
         mut exec: E,
         handle: R::Handle,
@@ -1255,4 +1300,38 @@ mod tests {
         let handle = TokioHandle;
         search_ipv6_name_parse_fails_test::<Runtime, TokioRuntime>(io_loop, handle);
     }
+
+    #[test]
+    fn test_build_names_onion() {
+        let handle = TokioHandle;
+        let mut config = ResolverConfig::default();
+        config.add_search(Name::from_ascii("example.com.").unwrap());
+        let resolver =
+            AsyncResolver::<GenericConnection, GenericConnectionProvider<TokioRuntime>>::new(
+                config,
+                ResolverOpts::default(),
+                handle,
+            )
+            .expect("failed to create resolver");
+        let tor_address = [
+            Name::from_ascii("2gzyxa5ihm7nsggfxnu52rck2vv4rvmdlkiu3zzui5du4xyclen53wid.onion")
+                .unwrap(),
+            Name::from_ascii("www.2gzyxa5ihm7nsggfxnu52rck2vv4rvmdlkiu3zzui5du4xyclen53wid.onion")
+                .unwrap(), // subdomain are allowed too
+        ];
+        let not_tor_address = [
+            Name::from_ascii("onion").unwrap(),
+            Name::from_ascii("www.onion").unwrap(),
+            Name::from_ascii("2gzyxa5ihm7nsggfxnu52rck2vv4rvmdlkiu3zzui5du4xyclen53wid.www.onion")
+                .unwrap(), // www before key
+            Name::from_ascii("2gzyxa5ihm7nsggfxnu52rck2vv4rvmdlkiu3zzui5du4xyclen53wid.onion.to")
+                .unwrap(), // Tor2web
+        ];
+        for name in &tor_address {
+            assert_eq!(resolver.build_names(name.clone()).len(), 1);
+        }
+        for name in &not_tor_address {
+            assert_eq!(resolver.build_names(name.clone()).len(), 2);
+        }
+    }
 }
