@@ -18,17 +18,17 @@ use std::time::Instant;
 use futures_util::future::Future;
 
 use proto::error::ProtoError;
-use proto::op::{Message, Query, ResponseCode};
+use proto::op::{Query, ResponseCode};
 use proto::rr::domain::usage::{
     ResolverUsage, DEFAULT, INVALID, IN_ADDR_ARPA_127, IP6_ARPA_1, LOCAL,
-    LOCALHOST as LOCALHOST_usage,
+    LOCALHOST as LOCALHOST_usage, ONION,
 };
 use proto::rr::rdata::SOA;
 use proto::rr::{DNSClass, Name, RData, Record, RecordType};
-use proto::xfer::{DnsHandle, DnsRequestOptions, DnsResponse};
+use proto::xfer::{DnsHandle, DnsRequestOptions, DnsResponse, FirstAnswer};
 
-use crate::dns_lru;
 use crate::dns_lru::DnsLru;
+use crate::dns_lru::{self, TtlConfig};
 use crate::error::*;
 use crate::lookup::Lookup;
 
@@ -80,7 +80,7 @@ where
     #[doc(hidden)]
     pub fn new(max_size: usize, client: C, preserve_intermediates: bool) -> Self {
         Self::with_cache(
-            DnsLru::new(max_size, Default::default()),
+            DnsLru::new(max_size, TtlConfig::default()),
             client,
             preserve_intermediates,
         )
@@ -88,7 +88,7 @@ where
 
     pub(crate) fn with_cache(lru: DnsLru, client: C, preserve_intermediates: bool) -> Self {
         let query_depth = Arc::new(AtomicU8::new(0));
-        CachingClient {
+        Self {
             lru,
             client,
             query_depth,
@@ -128,6 +128,7 @@ where
                 n if IP6_ARPA_1.zone_of(n) => &*LOCALHOST_usage,
                 n if INVALID.zone_of(n) => &*INVALID,
                 n if LOCAL.zone_of(n) => &*LOCAL,
+                n if ONION.zone_of(n) => &*ONION,
                 _ => &*DEFAULT,
             };
 
@@ -178,6 +179,7 @@ where
         let response_message = client
             .client
             .lookup(query.clone(), options)
+            .first_answer()
             .await
             .map_err(E::into);
 
@@ -207,8 +209,8 @@ where
                 Err(Self::handle_nxdomain(
                     is_dnssec,
                     false, /*tbd*/
-                    query,
-                    soa,
+                    *query,
+                    soa.map(|v| *v),
                     negative_ttl,
                     response_code,
                     trusted,
@@ -275,8 +277,8 @@ where
         if valid_nsec || !is_dnssec {
             // only trust if there were validated NSEC records
             ResolveErrorKind::NoRecordsFound {
-                query,
-                soa,
+                query: Box::new(query),
+                soa: soa.map(Box::new),
                 negative_ttl,
                 response_code,
                 trusted: true,
@@ -285,8 +287,8 @@ where
         } else {
             // not cacheable, no ttl...
             ResolveErrorKind::NoRecordsFound {
-                query,
-                soa,
+                query: Box::new(query),
+                soa: soa.map(Box::new),
                 negative_ttl: None,
                 response_code,
                 trusted,
@@ -328,11 +330,11 @@ where
                     // For SRV, the name added for the search becomes the target name.
                     //
                     // TODO: should this include the additionals?
-                    response.messages().flat_map(Message::answers).fold(
+                    response.answers().iter().fold(
                         (Cow::Borrowed(query.name()), INITIAL_TTL, false),
                         |(search_name, cname_ttl, was_cname), r| {
-                            match *r.rdata() {
-                                RData::CNAME(ref cname) => {
+                            match r.data() {
+                                Some(RData::CNAME(ref cname)) => {
                                     // take the minimum TTL of the cname_ttl and the next record in the chain
                                     let ttl = cname_ttl.min(r.ttl());
                                     debug_assert_eq!(r.rr_type(), RecordType::CNAME);
@@ -340,7 +342,7 @@ where
                                         return (Cow::Owned(cname.clone()), ttl, true);
                                     }
                                 }
-                                RData::SRV(ref srv) => {
+                                Some(RData::SRV(ref srv)) => {
                                     // take the minimum TTL of the cname_ttl and the next record in the chain
                                     let ttl = cname_ttl.min(r.ttl());
                                     debug_assert_eq!(r.rr_type(), RecordType::SRV);
@@ -357,18 +359,9 @@ where
                 };
 
             // take all answers. // TODO: following CNAMES?
-            let answers: Vec<Record> = response
-                .messages_mut()
-                .flat_map(Message::take_answers)
-                .collect();
-            let additionals: Vec<Record> = response
-                .messages_mut()
-                .flat_map(Message::take_additionals)
-                .collect();
-            let name_servers: Vec<Record> = response
-                .messages_mut()
-                .flat_map(Message::take_name_servers)
-                .collect();
+            let answers = response.take_answers();
+            let additionals = response.take_additionals();
+            let name_servers = response.take_name_servers();
 
             // set of names that still require resolution
             // TODO: this needs to be enhanced for SRV
@@ -439,7 +432,7 @@ where
         if was_cname && client.query_depth.load(Ordering::Acquire) < MAX_QUERY_DEPTH {
             let next_query = Query::query(search_name, query.query_type());
             Ok(Records::CnameChain {
-                next: Box::pin(CachingClient::inner_lookup(
+                next: Box::pin(Self::inner_lookup(
                     next_query,
                     options,
                     client.clone(),
@@ -480,6 +473,11 @@ where
             Err(err) => Err(self.lru.negative(query, err, Instant::now())),
         }
     }
+
+    /// Flushes/Removes all entries from the cache
+    pub fn clear_cache(&mut self) {
+        self.lru.clear();
+    }
 }
 
 enum Records {
@@ -519,14 +517,14 @@ mod tests {
             ..
         } = block_on(CachingClient::inner_lookup(
             Query::new(),
-            Default::default(),
+            DnsRequestOptions::default(),
             client,
             vec![],
         ))
         .unwrap_err()
         .kind()
         {
-            assert_eq!(*query, Query::new());
+            assert_eq!(**query, Query::new());
             assert_eq!(*negative_ttl, None);
         } else {
             panic!("wrong error received")
@@ -555,7 +553,7 @@ mod tests {
 
         let ips = block_on(CachingClient::inner_lookup(
             Query::new(),
-            Default::default(),
+            DnsRequestOptions::default(),
             client,
             vec![],
         ))
@@ -576,7 +574,7 @@ mod tests {
 
         let ips = block_on(CachingClient::inner_lookup(
             Query::new(),
-            Default::default(),
+            DnsRequestOptions::default(),
             client,
             vec![],
         ))
@@ -593,7 +591,7 @@ mod tests {
 
         let ips = block_on(CachingClient::inner_lookup(
             Query::new(),
-            Default::default(),
+            DnsRequestOptions::default(),
             client,
             vec![],
         ))
@@ -649,7 +647,7 @@ mod tests {
 
         let ips = block_on(CachingClient::inner_lookup(
             Query::query(Name::from_str("www.example.com.").unwrap(), query_type),
-            Default::default(),
+            DnsRequestOptions::default(),
             client,
             vec![],
         ))
@@ -684,7 +682,7 @@ mod tests {
                 Name::from_str("_443._tcp.www.example.com.").unwrap(),
                 RecordType::SRV,
             ),
-            Default::default(),
+            DnsRequestOptions::default(),
             client,
             vec![],
         ))
@@ -732,7 +730,7 @@ mod tests {
                 Name::from_str("_443._tcp.www.example.com.").unwrap(),
                 RecordType::SRV,
             ),
-            Default::default(),
+            DnsRequestOptions::default(),
             client,
             vec![],
         ))
@@ -821,7 +819,7 @@ mod tests {
 
         let records = CachingClient::handle_noerror(
             &mut client,
-            Default::default(),
+            DnsRequestOptions::default(),
             false,
             &Query::query(Name::from_str("ttl.example.com.").unwrap(), RecordType::A),
             message.into(),
@@ -858,7 +856,7 @@ mod tests {
 
         {
             let query = Query::query(Name::from_ascii("localhost.").unwrap(), RecordType::A);
-            let lookup = block_on(client.lookup(query.clone(), Default::default()))
+            let lookup = block_on(client.lookup(query.clone(), DnsRequestOptions::default()))
                 .expect("should have returned localhost");
             assert_eq!(lookup.query(), &query);
             assert_eq!(
@@ -869,7 +867,7 @@ mod tests {
 
         {
             let query = Query::query(Name::from_ascii("localhost.").unwrap(), RecordType::AAAA);
-            let lookup = block_on(client.lookup(query.clone(), Default::default()))
+            let lookup = block_on(client.lookup(query.clone(), DnsRequestOptions::default()))
                 .expect("should have returned localhost");
             assert_eq!(lookup.query(), &query);
             assert_eq!(
@@ -880,7 +878,7 @@ mod tests {
 
         {
             let query = Query::query(Name::from(Ipv4Addr::new(127, 0, 0, 1)), RecordType::PTR);
-            let lookup = block_on(client.lookup(query.clone(), Default::default()))
+            let lookup = block_on(client.lookup(query.clone(), DnsRequestOptions::default()))
                 .expect("should have returned localhost");
             assert_eq!(lookup.query(), &query);
             assert_eq!(
@@ -894,7 +892,7 @@ mod tests {
                 Name::from(Ipv6Addr::new(0, 0, 0, 0, 0, 0, 0, 1)),
                 RecordType::PTR,
             );
-            let lookup = block_on(client.lookup(query.clone(), Default::default()))
+            let lookup = block_on(client.lookup(query.clone(), DnsRequestOptions::default()))
                 .expect("should have returned localhost");
             assert_eq!(lookup.query(), &query);
             assert_eq!(
@@ -905,13 +903,13 @@ mod tests {
 
         assert!(block_on(client.lookup(
             Query::query(Name::from_ascii("localhost.").unwrap(), RecordType::MX),
-            Default::default()
+            DnsRequestOptions::default()
         ))
         .is_err());
 
         assert!(block_on(client.lookup(
             Query::query(Name::from(Ipv4Addr::new(127, 0, 0, 1)), RecordType::MX),
-            Default::default()
+            DnsRequestOptions::default()
         ))
         .is_err());
 
@@ -920,7 +918,7 @@ mod tests {
                 Name::from(Ipv6Addr::new(0, 0, 0, 0, 0, 0, 0, 1)),
                 RecordType::MX
             ),
-            Default::default()
+            DnsRequestOptions::default()
         ))
         .is_err());
     }
@@ -936,7 +934,7 @@ mod tests {
                 Name::from_ascii("horrible.invalid.").unwrap(),
                 RecordType::A,
             ),
-            Default::default()
+            DnsRequestOptions::default()
         ))
         .is_err());
     }
@@ -964,7 +962,7 @@ mod tests {
                 Name::from_ascii("www.example.local.").unwrap(),
                 RecordType::A,
             ),
-            Default::default()
+            DnsRequestOptions::default()
         ))
         .is_ok());
     }
