@@ -41,7 +41,7 @@ pub struct ResolverConfig {
 impl ResolverConfig {
     /// Creates a new empty configuration
     pub fn new() -> Self {
-        ResolverConfig {
+        Self {
             // TODO: this should get the hostname and use the basename as the default
             domain: None,
             search: vec![],
@@ -55,7 +55,7 @@ impl ResolverConfig {
     ///
     /// NameServerConfigGroups can be combined to use a set of different providers, see `NameServerConfigGroup` and `ResolverConfig::from_parts`
     pub fn google() -> Self {
-        ResolverConfig {
+        Self {
             // TODO: this should get the hostname and use the basename as the default
             domain: None,
             search: vec![],
@@ -65,11 +65,11 @@ impl ResolverConfig {
 
     /// Creates a default configuration, using `1.1.1.1`, `1.0.0.1` and `2606:4700:4700::1111`, `2606:4700:4700::1001` (thank you, Cloudflare).
     ///
-    /// Please see: https://www.cloudflare.com/dns/
+    /// Please see: <https://www.cloudflare.com/dns/>
     ///
     /// NameServerConfigGroups can be combined to use a set of different providers, see `NameServerConfigGroup` and `ResolverConfig::from_parts`
     pub fn cloudflare() -> Self {
-        ResolverConfig {
+        Self {
             // TODO: this should get the hostname and use the basename as the default
             domain: None,
             search: vec![],
@@ -79,12 +79,13 @@ impl ResolverConfig {
 
     /// Creates a configuration, using `1.1.1.1`, `1.0.0.1` and `2606:4700:4700::1111`, `2606:4700:4700::1001` (thank you, Cloudflare). This limits the registered connections to just TLS lookups
     ///
-    /// Please see: https://www.cloudflare.com/dns/
+    /// Please see: <https://www.cloudflare.com/dns/>
     ///
     /// NameServerConfigGroups can be combined to use a set of different providers, see `NameServerConfigGroup` and `ResolverConfig::from_parts`
     #[cfg(feature = "dns-over-tls")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "dns-over-tls")))]
     pub fn cloudflare_tls() -> Self {
-        ResolverConfig {
+        Self {
             // TODO: this should get the hostname and use the basename as the default
             domain: None,
             search: vec![],
@@ -94,12 +95,13 @@ impl ResolverConfig {
 
     /// Creates a configuration, using `1.1.1.1`, `1.0.0.1` and `2606:4700:4700::1111`, `2606:4700:4700::1001` (thank you, Cloudflare). This limits the registered connections to just HTTPS lookups
     ///
-    /// Please see: https://www.cloudflare.com/dns/
+    /// Please see: <https://www.cloudflare.com/dns/>
     ///
     /// NameServerConfigGroups can be combined to use a set of different providers, see `NameServerConfigGroup` and `ResolverConfig::from_parts`
     #[cfg(feature = "dns-over-https")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "dns-over-https")))]
     pub fn cloudflare_https() -> Self {
-        ResolverConfig {
+        Self {
             // TODO: this should get the hostname and use the basename as the default
             domain: None,
             search: vec![],
@@ -109,11 +111,11 @@ impl ResolverConfig {
 
     /// Creates a configuration, using `9.9.9.9`, `149.112.112.112` and `2620:fe::fe`, `2620:fe::fe:9`, the "secure" variants of the quad9 settings (thank you, Quad9).
     ///
-    /// Please see: https://www.quad9.net/faq/
+    /// Please see: <https://www.quad9.net/faq/>
     ///
     /// NameServerConfigGroups can be combined to use a set of different providers, see `NameServerConfigGroup` and `ResolverConfig::from_parts`
     pub fn quad9() -> Self {
-        ResolverConfig {
+        Self {
             // TODO: this should get the hostname and use the basename as the default
             domain: None,
             search: vec![],
@@ -123,12 +125,13 @@ impl ResolverConfig {
 
     /// Creates a configuration, using `9.9.9.9`, `149.112.112.112` and `2620:fe::fe`, `2620:fe::fe:9`, the "secure" variants of the quad9 settings. This limits the registered connections to just TLS lookups
     ///
-    /// Please see: https://www.quad9.net/faq/
+    /// Please see: <https://www.quad9.net/faq/>
     ///
     /// NameServerConfigGroups can be combined to use a set of different providers, see `NameServerConfigGroup` and `ResolverConfig::from_parts`
     #[cfg(feature = "dns-over-tls")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "dns-over-tls")))]
     pub fn quad9_tls() -> Self {
-        ResolverConfig {
+        Self {
             // TODO: this should get the hostname and use the basename as the default
             domain: None,
             search: vec![],
@@ -138,12 +141,13 @@ impl ResolverConfig {
 
     /// Creates a configuration, using `9.9.9.9`, `149.112.112.112` and `2620:fe::fe`, `2620:fe::fe:9`, the "secure" variants of the quad9 settings. This limits the registered connections to just HTTPS lookups
     ///
-    /// Please see: https://www.quad9.net/faq/
+    /// Please see: <https://www.quad9.net/faq/>
     ///
     /// NameServerConfigGroups can be combined to use a set of different providers, see `NameServerConfigGroup` and `ResolverConfig::from_parts`
     #[cfg(feature = "dns-over-https")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "dns-over-https")))]
     pub fn quad9_https() -> Self {
-        ResolverConfig {
+        Self {
             // TODO: this should get the hostname and use the basename as the default
             domain: None,
             search: vec![],
@@ -163,7 +167,7 @@ impl ResolverConfig {
         search: Vec<Name>,
         name_servers: G,
     ) -> Self {
-        ResolverConfig {
+        Self {
             domain,
             search,
             name_servers: name_servers.into(),
@@ -208,32 +212,43 @@ impl ResolverConfig {
 
     /// return the associated TlsClientConfig
     #[cfg(feature = "dns-over-rustls")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "dns-over-rustls")))]
     pub fn client_config(&self) -> &Option<TlsClientConfig> {
         &self.name_servers.1
     }
 
-    /// adds the [`rustls::ClientConf`] for every configured NameServer
+    /// adds the `rustls::ClientConf` for every configured NameServer
     /// of the Resolver.
     ///
     /// ```
     /// use std::sync::Arc;
     ///
-    /// use rustls::{ClientConfig, ProtocolVersion, RootCertStore};
+    /// use rustls::{ClientConfig, ProtocolVersion, RootCertStore, OwnedTrustAnchor};
     /// use trust_dns_resolver::config::ResolverConfig;
     /// use webpki_roots;
     ///
     /// let mut root_store = RootCertStore::empty();
-    /// root_store.add_server_trust_anchors(&webpki_roots::TLS_SERVER_ROOTS);
-    /// let versions = vec![ProtocolVersion::TLSv1_2];
+    /// root_store.add_server_trust_anchors(webpki_roots::TLS_SERVER_ROOTS.0.iter().map(|ta| {
+    ///     OwnedTrustAnchor::from_subject_spki_name_constraints(
+    ///         ta.subject,
+    ///         ta.spki,
+    ///         ta.name_constraints,
+    ///     )
+    /// }));
     ///
-    /// let mut client_config = ClientConfig::new();
-    /// client_config.root_store = root_store;
-    /// client_config.versions = versions;
+    /// let mut client_config = ClientConfig::builder()
+    ///     .with_safe_default_cipher_suites()
+    ///     .with_safe_default_kx_groups()
+    ///     .with_protocol_versions(&[&rustls::version::TLS12])
+    ///     .unwrap()
+    ///     .with_root_certificates(root_store)
+    ///     .with_no_client_auth();
     ///
     /// let mut resolver_config = ResolverConfig::quad9_tls();
     /// resolver_config.set_tls_client_config(Arc::new(client_config));
     /// ```
     #[cfg(feature = "dns-over-rustls")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "dns-over-rustls")))]
     pub fn set_tls_client_config(&mut self, client_config: Arc<ClientConfig>) {
         self.name_servers = self.name_servers.clone().with_client_config(client_config);
     }
@@ -244,7 +259,7 @@ impl Default for ResolverConfig {
     ///
     /// Please see Google's [privacy statement](https://developers.google.com/speed/public-dns/privacy) for important information about what they track, many ISP's track similar information in DNS. To use the system configuration see: `Resolver::from_system_conf` and `AsyncResolver::from_system_conf`
     fn default() -> Self {
-        ResolverConfig::google()
+        Self::google()
     }
 }
 
@@ -255,6 +270,7 @@ impl Default for ResolverConfig {
     derive(Serialize, Deserialize),
     serde(rename_all = "lowercase")
 )]
+#[non_exhaustive]
 pub enum Protocol {
     /// UDP is the traditional DNS port, this is generally the correct choice
     Udp,
@@ -262,12 +278,15 @@ pub enum Protocol {
     Tcp,
     /// Tls for DNS over TLS
     #[cfg(feature = "dns-over-tls")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "dns-over-tls")))]
     Tls,
     /// Https for DNS over HTTPS
     #[cfg(feature = "dns-over-https")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "dns-over-https")))]
     Https,
     /// mDNS protocol for performing multicast lookups
     #[cfg(feature = "mdns")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "mdns")))]
     Mdns,
 }
 
@@ -284,7 +303,7 @@ impl fmt::Display for Protocol {
             Protocol::Mdns => "mdns",
         };
 
-        write!(f, "{}", protocol)
+        f.write_str(protocol)
     }
 }
 
@@ -326,14 +345,15 @@ impl Protocol {
 impl Default for Protocol {
     /// Default protocol should be UDP, which is supported by all DNS servers
     fn default() -> Self {
-        Protocol::Udp
+        Self::Udp
     }
 }
 
-#[cfg(feature = "dns-over-rustls")]
-#[derive(Clone)]
 /// a compatibility wrapper around rustls
 /// ClientConfig
+#[cfg(feature = "dns-over-rustls")]
+#[cfg_attr(docsrs, doc(cfg(feature = "dns-over-rustls")))]
+#[derive(Clone)]
 pub struct TlsClientConfig(pub Arc<ClientConfig>);
 
 #[cfg(feature = "dns-over-rustls")]
@@ -365,16 +385,25 @@ pub struct NameServerConfig {
     /// SPKI name, only relevant for TLS connections
     #[cfg_attr(feature = "serde-config", serde(default))]
     pub tls_dns_name: Option<String>,
-    /// Default to not trust negative responses from upstream nameservers
+    /// Whether to trust `NXDOMAIN` responses from upstream nameservers.
     ///
-    /// When a SERVFAIL, NXDOMAIN and NoError/Empty response is received, the query will be
-    /// retried against other configured name servers.
+    /// When this is `true`, and an empty `NXDOMAIN` response is received, the
+    /// query will not be retried against other configured name servers.
+    ///
+    /// (On an empty `NoError` response, or a response with any other error
+    /// response code, the query will still be retried regardless of this
+    /// configuration setting.)
+    ///
+    /// Defaults to false.
     #[cfg_attr(feature = "serde-config", serde(default))]
     pub trust_nx_responses: bool,
     #[cfg(feature = "dns-over-rustls")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "dns-over-rustls")))]
     #[cfg_attr(feature = "serde-config", serde(skip))]
     /// optional configuration for the tls client
     pub tls_config: Option<TlsClientConfig>,
+    /// The client address (IP and port) to use for connecting to the server.
+    pub bind_addr: Option<SocketAddr>,
 }
 
 impl fmt::Display for NameServerConfig {
@@ -430,7 +459,7 @@ impl NameServerConfigGroup {
 
     /// Creates a new `NameServiceConfigGroup` with the specified capacity
     pub fn with_capacity(capacity: usize) -> Self {
-        NameServerConfigGroup(
+        Self(
             Vec::with_capacity(capacity),
             #[cfg(feature = "dns-over-rustls")]
             None,
@@ -451,6 +480,7 @@ impl NameServerConfigGroup {
                 trust_nx_responses,
                 #[cfg(feature = "dns-over-rustls")]
                 tls_config: None,
+                bind_addr: None,
             };
             let tcp = NameServerConfig {
                 socket_addr: SocketAddr::new(*ip, port),
@@ -459,6 +489,7 @@ impl NameServerConfigGroup {
                 trust_nx_responses,
                 #[cfg(feature = "dns-over-rustls")]
                 tls_config: None,
+                bind_addr: None,
             };
 
             name_servers.push(udp);
@@ -488,6 +519,7 @@ impl NameServerConfigGroup {
                 trust_nx_responses,
                 #[cfg(feature = "dns-over-rustls")]
                 tls_config: None,
+                bind_addr: None,
             };
 
             name_servers.push(config);
@@ -500,6 +532,7 @@ impl NameServerConfigGroup {
     ///
     /// This will create a TLS connections.
     #[cfg(feature = "dns-over-tls")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "dns-over-tls")))]
     pub fn from_ips_tls(
         ips: &[IpAddr],
         port: u16,
@@ -513,6 +546,7 @@ impl NameServerConfigGroup {
     ///
     /// This will create a HTTPS connections.
     #[cfg(feature = "dns-over-https")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "dns-over-https")))]
     pub fn from_ips_https(
         ips: &[IpAddr],
         port: u16,
@@ -533,52 +567,57 @@ impl NameServerConfigGroup {
     ///
     /// Please see Google's [privacy statement](https://developers.google.com/speed/public-dns/privacy) for important information about what they track, many ISP's track similar information in DNS. To use the system configuration see: `Resolver::from_system_conf` and `AsyncResolver::from_system_conf`
     #[cfg(feature = "dns-over-https")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "dns-over-https")))]
     pub fn google_https() -> Self {
         Self::from_ips_https(GOOGLE_IPS, 53, "dns.google".to_string(), true)
     }
 
     /// Creates a default configuration, using `1.1.1.1`, `1.0.0.1` and `2606:4700:4700::1111`, `2606:4700:4700::1001` (thank you, Cloudflare).
     ///
-    /// Please see: https://www.cloudflare.com/dns/
+    /// Please see: <https://www.cloudflare.com/dns/>
     pub fn cloudflare() -> Self {
         Self::from_ips_clear(CLOUDFLARE_IPS, 53, true)
     }
 
     /// Creates a configuration, using `1.1.1.1`, `1.0.0.1` and `2606:4700:4700::1111`, `2606:4700:4700::1001` (thank you, Cloudflare). This limits the registered connections to just TLS lookups
     ///
-    /// Please see: https://www.cloudflare.com/dns/
+    /// Please see: <https://www.cloudflare.com/dns/>
     #[cfg(feature = "dns-over-tls")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "dns-over-tls")))]
     pub fn cloudflare_tls() -> Self {
         Self::from_ips_tls(CLOUDFLARE_IPS, 853, "cloudflare-dns.com".to_string(), true)
     }
 
     /// Creates a configuration, using `1.1.1.1`, `1.0.0.1` and `2606:4700:4700::1111`, `2606:4700:4700::1001` (thank you, Cloudflare). This limits the registered connections to just HTTPS lookups
     ///
-    /// Please see: https://www.cloudflare.com/dns/
+    /// Please see: <https://www.cloudflare.com/dns/>
     #[cfg(feature = "dns-over-https")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "dns-over-https")))]
     pub fn cloudflare_https() -> Self {
         Self::from_ips_https(CLOUDFLARE_IPS, 443, "cloudflare-dns.com".to_string(), true)
     }
 
     /// Creates a configuration, using `9.9.9.9`, `149.112.112.112` and `2620:fe::fe`, `2620:fe::fe:9`, the "secure" variants of the quad9 settings (thank you, Quad9).
     ///
-    /// Please see: https://www.quad9.net/faq/
+    /// Please see: <https://www.quad9.net/faq/>
     pub fn quad9() -> Self {
         Self::from_ips_clear(QUAD9_IPS, 53, true)
     }
 
     /// Creates a configuration, using `9.9.9.9`, `149.112.112.112` and `2620:fe::fe`, `2620:fe::fe:9`, the "secure" variants of the quad9 settings. This limits the registered connections to just TLS lookups
     ///
-    /// Please see: https://www.quad9.net/faq/
+    /// Please see: <https://www.quad9.net/faq/>
     #[cfg(feature = "dns-over-tls")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "dns-over-tls")))]
     pub fn quad9_tls() -> Self {
         Self::from_ips_tls(QUAD9_IPS, 853, "dns.quad9.net".to_string(), true)
     }
 
     /// Creates a configuration, using `9.9.9.9`, `149.112.112.112` and `2620:fe::fe`, `2620:fe::fe:9`, the "secure" variants of the quad9 settings. This limits the registered connections to just HTTPS lookups
     ///
-    /// Please see: https://www.quad9.net/faq/
+    /// Please see: <https://www.quad9.net/faq/>
     #[cfg(feature = "dns-over-https")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "dns-over-https")))]
     pub fn quad9_https() -> Self {
         Self::from_ips_https(QUAD9_IPS, 443, "dns.quad9.net".to_string(), true)
     }
@@ -608,11 +647,20 @@ impl NameServerConfigGroup {
         }
     }
 
-    #[cfg(feature = "dns-over-rustls")]
     /// add a [`rustls::ClientConfig`]
+    #[cfg(feature = "dns-over-rustls")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "dns-over-rustls")))]
     pub fn with_client_config(self, client_config: Arc<ClientConfig>) -> Self {
         Self(self.0, Some(TlsClientConfig(client_config)))
     }
+
+    /// Sets the client address (IP and port) to connect from on all name servers.
+    pub fn with_bind_addr(mut self, bind_addr: Option<SocketAddr>) -> Self {
+        for server in &mut self.0 {
+            server.bind_addr = bind_addr;
+        }
+        self
+    }
 }
 
 impl Default for NameServerConfigGroup {
@@ -638,11 +686,11 @@ impl From<Vec<NameServerConfig>> for NameServerConfigGroup {
     fn from(configs: Vec<NameServerConfig>) -> Self {
         #[cfg(not(feature = "dns-over-rustls"))]
         {
-            NameServerConfigGroup(configs)
+            Self(configs)
         }
         #[cfg(feature = "dns-over-rustls")]
         {
-            NameServerConfigGroup(configs, None)
+            Self(configs, None)
         }
     }
 }
@@ -666,7 +714,7 @@ pub enum LookupIpStrategy {
 impl Default for LookupIpStrategy {
     /// Returns [`LookupIpStrategy::Ipv4thenIpv6`] as the default.
     fn default() -> Self {
-        LookupIpStrategy::Ipv4thenIpv6
+        Self::Ipv4thenIpv6
     }
 }
 
@@ -678,6 +726,7 @@ impl Default for LookupIpStrategy {
     serde(default)
 )]
 #[allow(dead_code)] // TODO: remove after all params are supported
+#[non_exhaustive]
 pub struct ResolverOpts {
     /// Sets the number of dots that must appear (unless it's a final dot representing the root)
     ///  that must appear before a query is assumed to include the TLD. The default is one, which
@@ -734,6 +783,8 @@ pub struct ResolverOpts {
     pub num_concurrent_reqs: usize,
     /// Preserve all intermediate records in the lookup response, suchas CNAME records
     pub preserve_intermediates: bool,
+    /// Try queries over TCP if they fail over UDP.
+    pub try_tcp_on_error: bool,
 }
 
 impl Default for ResolverOpts {
@@ -741,7 +792,7 @@ impl Default for ResolverOpts {
     ///
     /// This follows the resolv.conf defaults as defined in the [Linux man pages](http://man7.org/linux/man-pages/man5/resolv.conf.5.html)
     fn default() -> Self {
-        ResolverOpts {
+        Self {
             ndots: 1,
             timeout: Duration::from_secs(5),
             attempts: 2,
@@ -758,6 +809,7 @@ impl Default for ResolverOpts {
             negative_max_ttl: None,
             num_concurrent_reqs: 2,
             preserve_intermediates: false,
+            try_tcp_on_error: false,
         }
     }
 }
