@@ -62,7 +62,7 @@ impl LookupIp {
 
 impl From<Lookup> for LookupIp {
     fn from(lookup: Lookup) -> Self {
-        LookupIp(lookup)
+        Self(lookup)
     }
 }
 
@@ -117,7 +117,7 @@ impl Iterator for LookupIpIntoIter {
     }
 }
 
-/// The Future returned from [`AsyncResolver`] when performing an A or AAAA lookup.
+/// The Future returned from [crate::AsyncResolver] when performing an A or AAAA lookup.
 ///
 /// This type isn't necessarily something that should be used by users, see the default TypeParameters are generally correct
 pub struct LookupIpFuture<C, E>
@@ -216,7 +216,7 @@ where
     ) -> Self {
         let empty =
             ResolveError::from(ResolveErrorKind::Message("can not lookup IPs for no names"));
-        LookupIpFuture {
+        Self {
             names,
             strategy,
             client_cache,
@@ -452,12 +452,14 @@ pub mod tests {
     use std::sync::{Arc, Mutex};
 
     use futures_executor::block_on;
-    use futures_util::{future, future::Future};
+    use futures_util::future;
 
     use proto::op::Message;
     use proto::rr::{Name, RData, Record};
     use proto::xfer::{DnsHandle, DnsRequest, DnsResponse};
 
+    use futures_util::stream::{once, Stream};
+
     use super::*;
     use crate::error::ResolveError;
 
@@ -468,13 +470,13 @@ pub mod tests {
 
     impl DnsHandle for MockDnsHandle {
         type Response =
-            Pin<Box<dyn Future<Output = Result<DnsResponse, ResolveError>> + Send + Unpin>>;
+            Pin<Box<dyn Stream<Item = Result<DnsResponse, ResolveError>> + Send + Unpin>>;
         type Error = ResolveError;
 
         fn send<R: Into<DnsRequest>>(&mut self, _: R) -> Self::Response {
-            Box::pin(future::ready(
+            Box::pin(once(future::ready(
                 self.messages.lock().unwrap().pop().unwrap_or_else(empty),
-            ))
+            )))
         }
     }
 
@@ -526,7 +528,7 @@ pub mod tests {
             block_on(ipv4_only(
                 Name::root(),
                 CachingClient::new(0, mock(vec![v4_message()]), false),
-                Default::default(),
+                DnsRequestOptions::default(),
                 None,
             ))
             .unwrap()
@@ -543,7 +545,7 @@ pub mod tests {
             block_on(ipv6_only(
                 Name::root(),
                 CachingClient::new(0, mock(vec![v6_message()]), false),
-                Default::default(),
+                DnsRequestOptions::default(),
                 None,
             ))
             .unwrap()
@@ -562,7 +564,7 @@ pub mod tests {
             block_on(ipv4_and_ipv6(
                 Name::root(),
                 CachingClient::new(0, mock(vec![v6_message(), v4_message()]), false),
-                Default::default(),
+                DnsRequestOptions::default(),
                 None,
             ))
             .unwrap()
@@ -580,7 +582,7 @@ pub mod tests {
             block_on(ipv4_and_ipv6(
                 Name::root(),
                 CachingClient::new(0, mock(vec![empty(), v4_message()]), false),
-                Default::default(),
+                DnsRequestOptions::default(),
                 None,
             ))
             .unwrap()
@@ -595,7 +597,7 @@ pub mod tests {
             block_on(ipv4_and_ipv6(
                 Name::root(),
                 CachingClient::new(0, mock(vec![error(), v4_message()]), false),
-                Default::default(),
+                DnsRequestOptions::default(),
                 None,
             ))
             .unwrap()
@@ -610,7 +612,7 @@ pub mod tests {
             block_on(ipv4_and_ipv6(
                 Name::root(),
                 CachingClient::new(0, mock(vec![v6_message(), empty()]), false),
-                Default::default(),
+                DnsRequestOptions::default(),
                 None,
             ))
             .unwrap()
@@ -625,7 +627,7 @@ pub mod tests {
             block_on(ipv4_and_ipv6(
                 Name::root(),
                 CachingClient::new(0, mock(vec![v6_message(), error()]), false),
-                Default::default(),
+                DnsRequestOptions::default(),
                 None,
             ))
             .unwrap()
@@ -643,7 +645,7 @@ pub mod tests {
             block_on(ipv6_then_ipv4(
                 Name::root(),
                 CachingClient::new(0, mock(vec![v6_message()]), false),
-                Default::default(),
+                DnsRequestOptions::default(),
                 None,
             ))
             .unwrap()
@@ -658,7 +660,7 @@ pub mod tests {
             block_on(ipv6_then_ipv4(
                 Name::root(),
                 CachingClient::new(0, mock(vec![v4_message(), empty()]), false),
-                Default::default(),
+                DnsRequestOptions::default(),
                 None,
             ))
             .unwrap()
@@ -673,7 +675,7 @@ pub mod tests {
             block_on(ipv6_then_ipv4(
                 Name::root(),
                 CachingClient::new(0, mock(vec![v4_message(), error()]), false),
-                Default::default(),
+                DnsRequestOptions::default(),
                 None,
             ))
             .unwrap()
@@ -691,7 +693,7 @@ pub mod tests {
             block_on(ipv4_then_ipv6(
                 Name::root(),
                 CachingClient::new(0, mock(vec![v4_message()]), false),
-                Default::default(),
+                DnsRequestOptions::default(),
                 None,
             ))
             .unwrap()
@@ -706,7 +708,7 @@ pub mod tests {
             block_on(ipv4_then_ipv6(
                 Name::root(),
                 CachingClient::new(0, mock(vec![v6_message(), empty()]), false),
-                Default::default(),
+                DnsRequestOptions::default(),
                 None,
             ))
             .unwrap()
@@ -721,7 +723,7 @@ pub mod tests {
             block_on(ipv4_then_ipv6(
                 Name::root(),
                 CachingClient::new(0, mock(vec![v6_message(), error()]), false),
-                Default::default(),
+                DnsRequestOptions::default(),
                 None,
             ))
             .unwrap()
